#!/usr/bin/python2.5
# rapier: a small bible reader for maemo.
# Copyright 2007 Pierre Amadio 
# pierre.amadio@libertysurf.fr
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation version 2.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

"""
[Install]
DataPath=/usr/share/sword/

http://developer.gnome.org/doc/API/gnome-vfs/
"""
import Sword
from gnome import gnomevfs
import os
import tarfile
import re

def get_config():
    sword_mgr=Sword.SWMgr()
    sword_conf_path=sword_mgr.globalConfPath

    uri=gnomevfs.URI(sword_conf_path)
    sword_conf=gnomevfs.Handle(uri,open_mode=gnomevfs.OPEN_READ)
    size=sword_conf.get_file_info().size
    content=sword_conf.read(size)
    sword_conf.close()
    return content

def get_DataPath():
    sword_mgr=Sword.SWMgr()
    return sword_mgr.prefixPath

def set_DataPath(value):
    sword_mgr=Sword.SWMgr()
    sword_conf_path=sword_mgr.globalConfPath

    uri=gnomevfs.URI(sword_conf_path)
    try:
        sword_conf=gnomevfs.Handle(uri,open_mode=gnomevfs.OPEN_WRITE)
    except gnomevfs.Error,fault:
        print "ouuups, probelm setting conf:",fault
        return fault
    
    sword_conf.write("[Install]\nDataPath=%s"%value)
    sword_conf.close()

def change_DataPath(src,dst):
    src_uri=gnomevfs.URI(src)
    dst_uri=gnomevfs.URI(dst)
    gnomevfs.xfer_uri_list([src_uri],[dst_uri],\
                           gnomevfs.XFER_RECURSIVE,\
                           gnomevfs.XFER_ERROR_MODE_ABORT,\
                           gnomevfs.XFER_OVERWRITE_ACTION_REPLACE_ALL)
    set_DataPath(dst)
    
def has_clucene_index(modName):
    mgr=Sword.SWMgr()
    a=mgr.getModule(modName)
    return a.isSearchOptimallySupported('plop',-4,1,Sword.VerseKey())

class InstallMgr:
    """
    The sword swing binding does not seems to make InstallMgr
    available, so here comes a small class to handle installation
    of remote Sword modules.
    """

    def __init__(self,location='ftp://crosswire.org/pub/sword/raw'):
        self.mgr=Sword.SWMgr()
        self.rep_location=location
        self.tmpDir=get_DataPath()+"tmp"
        if not os.path.isdir(self.tmpDir):
            os.mkdir(self.tmpDir)

    def remove_index(self,mod):
        swmod=self.mgr.getModule(mod)
        swmod.deleteSearchFramework()
        self.mgr=Sword.SWMgr()
        return True


    def create_index(self,mod):
        cmd="/usr/bin/mkfastmod %s"%mod
        pipe=os.popen(cmd,'r')
        pipe.close()
        self.mgr=Sword.SWMgr()
        return True

    def remove_module(self,mod):
        print "Let's remove '%s'"%mod
        cmd="/usr/bin/installmgr -u '%s'"%mod
        pipe=os.popen(cmd,'r')
        pipe.close()
        self.mgr=Sword.SWMgr()
        return True

        
    def install_module(self,dataPath,conf_file):
        """
        dataPath ./modules/texts/ztext/sparv/
        conf file sparv.conf

        Return True if installation succeed.
        Return False otherwise.
        
        """
        self.mgr=Sword.SWMgr()
        #print "dataPath='%s'"%dataPath
        modDirNameReg=re.compile("^.*/(\w+)/?\s*$")
        if modDirNameReg.search(dataPath):
            modDirName=modDirNameReg.search(dataPath).group(1)
            #print "modDirName='%s'"%modDirName

        else:
            print "Problem finding a directory name",dataPath

        #local_data_path=self.mgr.globalConfPath

        remote_conf=self.rep_location+"/mods.d/"+conf_file
        remote_conf_uri=gnomevfs.URI(remote_conf)
        remote_mod=self.rep_location+"/"+dataPath
        remote_mod_uri=gnomevfs.URI(remote_mod)

        tmp_conf=self.tmpDir+"/"+conf_file
        tmp_conf_uri=gnomevfs.URI(tmp_conf)
        tmp_mod=self.tmpDir+"/"+modDirName
        tmp_mod_uri=gnomevfs.URI(tmp_mod)


        dst_conf=self.mgr.prefixPath+"/mods.d/"+conf_file
        dst_conf_uri=gnomevfs.URI(dst_conf)
        dst_module=self.mgr.prefixPath+"/"+dataPath
        dst_module_uri=gnomevfs.URI(dst_module)

        #print "dst_conf='%s'"%dst_conf
        #print "dst_module='%s'"%dst_module

        if os.path.isdir(tmp_mod):
            pipe=os.popen("rm -rf %s"%tmp_mod,'r')
            pipe.close()
        if os.path.isfile(tmp_conf):
            os.remove(tmp_conf)
        problem=False


        """
        Copying in a temporary location.
        """
        #print "donwloading conf file"
        try:
            gnomevfs.xfer_uri(remote_conf_uri,tmp_conf_uri,
                              gnomevfs.XFER_DEFAULT,\
                              gnomevfs.XFER_ERROR_ACTION_ABORT,\
                              gnomevfs.XFER_OVERWRITE_ACTION_REPLACE_ALL\
                              )

        except:
            problem=True

        #print "donwloading module itself"
        try:
            gnomevfs.xfer_uri(remote_mod_uri,tmp_mod_uri,
                              gnomevfs.XFER_DEFAULT,\
                              gnomevfs.XFER_ERROR_ACTION_ABORT,\
                              gnomevfs.XFER_OVERWRITE_ACTION_REPLACE_ALL\
                              )
            
        except:
            problem=True


        if  problem:
            if os.path.isdir(tmp_mod):
                pipe=os.popen("rm -rf %s"%tmp_mod,'r')
                pipe.close()
            if os.path.isfile(tmp_conf):
                os.remove(tmp_conf)
            return False

        """
        Copying from the temporary location to the real one.
        """

        try:
            gnomevfs.xfer_uri(tmp_mod_uri,dst_module_uri,
                              gnomevfs.XFER_DEFAULT,\
                              gnomevfs.XFER_ERROR_ACTION_ABORT,\
                              gnomevfs.XFER_OVERWRITE_ACTION_REPLACE_ALL\
                              )
        except:
            problem=True


        try:
            gnomevfs.xfer_uri(tmp_conf_uri,dst_conf_uri,
                              gnomevfs.XFER_DEFAULT,\
                              gnomevfs.XFER_ERROR_ACTION_ABORT,\
                              gnomevfs.XFER_OVERWRITE_ACTION_REPLACE_ALL\
                              )
        except:
            problem=True

        if os.path.isdir(tmp_mod):
            pipe=os.popen("rm -rf %s"%tmp_mod,'r')
            pipe.close()
        if os.path.isfile(tmp_conf):
            os.remove(tmp_conf)



        self.mgr=Sword.SWMgr()
        return not problem
        
    def download_mods_list (self,src_uri,dst_uri):
        try:
            test=gnomevfs.Handle(src_uri,open_mode=gnomevfs.OPEN_READ)
            plop=test.get_file_info()
        except gnomevfs.Error,fault:
            print "problem=",fault
            return False
        except :
            print "Something is borked"
	    raise
            return False

        try:
            gnomevfs.xfer_uri(src_uri,dst_uri,
                              gnomevfs.XFER_DEFAULT,\
                              gnomevfs.XFER_ERROR_ACTION_ABORT,\
                              gnomevfs.XFER_OVERWRITE_ACTION_REPLACE_ALL)

        except:
            raise
        return True

    def untar_mods_list(self,dst):
        isTarfile=False
        try:
            isTarfile=tarfile.is_tarfile(dst)
        except :
            isTarfile=False

        if isTarfile:
            tar=tarfile.open(dst,'r')
            for tarinfo in tar:
                tar.extract(tarinfo,self.tmpDir)
        else:
            print "this is not a tarfile"
            return False

        tar.close()
        return True

    def get_module_info(self,dir,conf):
        filename="%s/%s"%(dir,conf.name)
        uri=gnomevfs.URI(filename)
        curFile=gnomevfs.Handle(uri,open_mode=gnomevfs.OPEN_READ)
        size=curFile.get_file_info().size
        content=curFile.read(size)
        curFile.close()
        lines=content.splitlines()
        namereg=re.compile("^\[(.*)\]\s?$")
        out={}
        curName=False
        for l in lines:
            kvreg=re.compile("^(.*)=(.*)$")
            if kvreg.search(l):
                curKey=kvreg.search(l).group(1)
                curValue=kvreg.search(l).group(2)
                out[curKey]=curValue

            if namereg.search(l):
                curName=namereg.search(l).group(1)


        if not curName:
            print "Problem finding module name for",conf.name

        out['Name']=curName        
        return out
        
    def get_remote_modules_list(self):
        tarfile_location=self.rep_location+"/mods.d.tar.gz"
        #tarfile_location="/etc/passwd"
        dst=self.tmpDir+"/mods.d.tar.gz"
        src_uri=gnomevfs.URI(tarfile_location)
        dst_uri=gnomevfs.URI(dst)

        all_modules=[]
        if not self.download_mods_list(src_uri,dst_uri):
            print "download does not work"
            return []


        if not self.untar_mods_list(dst):
            print "problem untarring modules list"
            return []
        

        modules_dir_uri=gnomevfs.URI(self.tmpDir+"/mods.d")
        modules_dir=gnomevfs.DirectoryHandle(modules_dir_uri)
        while True:
            try:
                curFileInfo=modules_dir.next()
                if not re.compile(".*conf$").search(curFileInfo.name):
                    continue
                
                cur_info=self.get_module_info(self.tmpDir+"/mods.d",\
                                              curFileInfo)
                cur_info['conf_file_name']=curFileInfo.name
                cur_uri=gnomevfs.URI(self.tmpDir+"/mods.d/"+curFileInfo.name)
                gnomevfs.unlink(cur_uri)

                """Don't need  commentary or non bible books"""
                if cur_info.has_key('Category'):
                    continue

                if cur_info.has_key('LCSH'):
                    snt=cur_info['LCSH']
                    if re.compile('Commentaries').search(snt):
                        continue
                    if re.compile('Dictionary').search(snt):
                        continue
                    if re.compile('Dictionaries').search(snt):
                        continue
                    if re.compile('Indexes').search(snt):
                        continue
                
                all_modules.append(cur_info)

            except StopIteration:
                break
            except:
                raise


        gnomevfs.unlink(dst_uri)
        newDirUri=gnomevfs.URI(self.tmpDir+"/mods.d")
        gnomevfs.remove_directory(newDirUri)



        return all_modules
